
#include <linux/soundcard.h>
#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <stdlib.h>
#include "actions.h"


int save_to_file(int mixer_fd) 
{
  StereoVolume vol;
  FILE *configfile;
  char homefile[100];
  if (getenv("HOME")) strcpy(homefile, (char *) getenv("HOME"));
  strcat(homefile,"/.mixrc");
    if ((configfile = fopen(homefile, "w")) == NULL) 
    {
      fprintf(stderr, "Cannot open file; aborting\n");
      exit(1);
    }
    printf("saving to file %s\n", homefile);
    ioctl(mixer_fd,MIXER_READ(SOUND_MIXER_VOLUME), &vol);
    fprintf(configfile, "%i\t; vol\n", vol.left);
    ioctl(mixer_fd,MIXER_READ(SOUND_MIXER_PCM), &vol);
    fprintf(configfile, "%i\t; pcm\n", vol.left);
    ioctl(mixer_fd,MIXER_READ(SOUND_MIXER_SPEAKER), &vol);
    fprintf(configfile, "%i\t; speaker\n", vol.left);
    ioctl(mixer_fd,MIXER_READ(SOUND_MIXER_CD), &vol);
    fprintf(configfile, "%i\t; cd\n",  vol.left);
    ioctl(mixer_fd,MIXER_READ(SOUND_MIXER_MIC), &vol);
    fprintf(configfile, "%i\t; mic\n", vol.left);
    ioctl(mixer_fd,MIXER_READ(SOUND_MIXER_LINE), &vol);
    fprintf(configfile, "%i\t; line\n", vol.left);
    fclose(configfile);
}


int load_from_file(int mixer_fd) 
{
  FILE *configfile;
  char homefile[90];
  char buf[90];
  StereoVolume vol;
  unsigned char cnt;
  if (getenv("HOME")) strcpy(homefile, (char *) getenv("HOME"));
  strcat(homefile,"/.mixrc");
  if ((configfile = fopen(homefile, "r")) == NULL) 
  {
    fprintf(stderr, "Cannot open file; aborting\n");
    return 1;
  }
  cnt=0;
  fread(buf,90,1,configfile);
  /* read in main vol */
  vol.right=vol.left=atoi(buf);
  ioctl(mixer_fd,MIXER_WRITE(SOUND_MIXER_VOLUME), &vol);
  /* read in pcm */
  while(buf[cnt++] != '\n'); 
  vol.right=vol.left=atoi(buf+cnt);
  ioctl(mixer_fd,MIXER_WRITE(SOUND_MIXER_PCM), &vol);
  /* read in speaker */
  while(buf[cnt++] != '\n'); 
  vol.right=vol.left=atoi(buf+cnt);
  ioctl(mixer_fd,MIXER_WRITE(SOUND_MIXER_SPEAKER), &vol);
  /* read in cd */
  while(buf[cnt++] != '\n'); 
  vol.right=vol.left=atoi(buf+cnt);
  ioctl(mixer_fd,MIXER_WRITE(SOUND_MIXER_CD), &vol);
  /* read in mic */
  while(buf[cnt++] != '\n'); 
  vol.right=vol.left=atoi(buf+cnt);
  ioctl(mixer_fd,MIXER_WRITE(SOUND_MIXER_MIC), &vol);
  /* read in line */
  while(buf[cnt++] != '\n'); 
  vol.right=vol.left=atoi(buf+cnt);
  ioctl(mixer_fd,MIXER_WRITE(SOUND_MIXER_LINE), &vol);
  fclose(configfile);

}

int args(int argc, char **argv) 
{
  StereoVolume vol;
  extern int mixer_fd;
  if(argc>2) {
    vol.left=vol.right=atoi(argv[2]);

    /* PCM VOL ADJUST SECTION */
    if (!strcmp(argv[1],"pcm")) 
    {
      if (argv[2][0]=='-') 
      {
        ioctl(mixer_fd,MIXER_READ(SOUND_MIXER_PCM), &vol);
        vol.left=vol.right=vol.left-atoi(argv[2]+1);
        ioctl(mixer_fd,MIXER_WRITE(SOUND_MIXER_PCM), &vol);
      } 
      else if (argv[2][0]=='+') 
      {
        ioctl(mixer_fd,MIXER_READ(SOUND_MIXER_PCM), &vol);
        vol.left=vol.right=vol.left+atoi(argv[2]+1);
        ioctl(mixer_fd,MIXER_WRITE(SOUND_MIXER_PCM), &vol);
      }
      else if (argv[2][0]>=48 && argv[2][0]<=57) 
        ioctl(mixer_fd,MIXER_WRITE(SOUND_MIXER_PCM), &vol);
      else printf("operation unable to be completed\n");
    } 
    
    /* SPEAKER VOL ADJUST SECTION */
    else if (!strcmp(argv[1],"speaker")) 
    {
      if (argv[2][0]=='-') 
      {
        ioctl(mixer_fd,MIXER_READ(SOUND_MIXER_SPEAKER), &vol);
        vol.left=vol.right=vol.left-atoi(argv[2]+1);
        ioctl(mixer_fd,MIXER_WRITE(SOUND_MIXER_SPEAKER), &vol);
      } 
      else if (argv[2][0]=='+') 
      {
        ioctl(mixer_fd,MIXER_READ(SOUND_MIXER_SPEAKER), &vol);
        vol.left=vol.right=vol.left+atoi(argv[2]+1);
        ioctl(mixer_fd,MIXER_WRITE(SOUND_MIXER_SPEAKER), &vol);
      }
      else if (argv[2][0]>=48 && argv[2][0]<=57) 
        ioctl(mixer_fd,MIXER_WRITE(SOUND_MIXER_SPEAKER), &vol);
      else printf("operation unable to be completed\n");
    }

    /* CD VOL ADJUST SECTION */
    else if (!strcmp(argv[1],"cd")) 
    {
      if (argv[2][0]=='-') 
      {
        ioctl(mixer_fd,MIXER_READ(SOUND_MIXER_CD), &vol);
        vol.left=vol.right=vol.left-atoi(argv[2]+1);
        ioctl(mixer_fd,MIXER_WRITE(SOUND_MIXER_CD), &vol);
      }
      else if (argv[2][0]=='+') 
      {
        ioctl(mixer_fd,MIXER_READ(SOUND_MIXER_CD), &vol);
        vol.left=vol.right=vol.left+atoi(argv[2]+1);
        ioctl(mixer_fd,MIXER_WRITE(SOUND_MIXER_CD), &vol);
      }
      else if (argv[2][0]>=48 && argv[2][0]<=57) 
        ioctl(mixer_fd,MIXER_WRITE(SOUND_MIXER_CD), &vol);
      else printf("operation unable to be completed\n");
    }

    /* MIC VOL ADJUST SECTION */
    else if (!strcmp(argv[1],"mic")) 
    {
      if (argv[2][0]=='-') 
      {
        ioctl(mixer_fd,MIXER_READ(SOUND_MIXER_MIC), &vol);
        vol.left=vol.right=vol.left-atoi(argv[2]+1);
        ioctl(mixer_fd,MIXER_WRITE(SOUND_MIXER_MIC), &vol);
      }
       else if (argv[2][0]=='+') 
      {
        ioctl(mixer_fd,MIXER_READ(SOUND_MIXER_MIC), &vol);
        vol.left=vol.right=vol.left+atoi(argv[2]+1);
        ioctl(mixer_fd,MIXER_WRITE(SOUND_MIXER_MIC), &vol);
      }
      else if (argv[2][0]>=48 && argv[2][0]<=57) 
        ioctl(mixer_fd,MIXER_WRITE(SOUND_MIXER_MIC), &vol);
      else printf("operation unable to be completed\n");
    }

    /* LINE VOL ADJUST SECTION */
    else if (!strcmp(argv[1],"line")) 
    {
      if (argv[2][0]=='-') 
      {
        ioctl(mixer_fd,MIXER_READ(SOUND_MIXER_LINE), &vol);
        vol.left=vol.right=vol.left-atoi(argv[2]+1);
        ioctl(mixer_fd,MIXER_WRITE(SOUND_MIXER_LINE), &vol);
      }
      else if (argv[2][0]=='+') 
      {
        ioctl(mixer_fd,MIXER_READ(SOUND_MIXER_LINE), &vol);
        vol.left=vol.right=vol.left+atoi(argv[2]+1);
        ioctl(mixer_fd,MIXER_WRITE(SOUND_MIXER_LINE), &vol);
      }
      else if (argv[2][0]>=48 && argv[2][0]<=57) 
        ioctl(mixer_fd,MIXER_WRITE(SOUND_MIXER_LINE), &vol);
      else printf("operation unable to be completed\n");
    }


    else 
      printf("option %s not recognized\n", argv[1]);
    

  } 
  else 
  {
    if (!strcmp(argv[1],"load")) 
    {
      load_from_file(mixer_fd);
    }
    else if (!strcmp(argv[1],"save")) 
    {
      save_to_file(mixer_fd);
    }
    else if (argv[1][0]=='-') 
    {
      ioctl(mixer_fd,MIXER_READ(SOUND_MIXER_VOLUME), &vol);
      vol.left=vol.right=vol.left-atoi(argv[1]+1);
      ioctl(mixer_fd,MIXER_WRITE(SOUND_MIXER_VOLUME), &vol);
    } 
    else if (argv[1][0]=='+') 
    {
      ioctl(mixer_fd,MIXER_READ(SOUND_MIXER_VOLUME), &vol);
      vol.left=vol.right=vol.left+atoi(argv[1]+1);
      ioctl(mixer_fd,MIXER_WRITE(SOUND_MIXER_VOLUME), &vol);
    }
    else if (!strcmp(argv[1],"list")) 
    {
      ioctl(mixer_fd,MIXER_READ(SOUND_MIXER_VOLUME), &vol);
      printf("vol: \t\t%i\n",vol.left);
      ioctl(mixer_fd,MIXER_READ(SOUND_MIXER_PCM), &vol);
      printf("pcm: \t\t%i\n",vol.left);
      ioctl(mixer_fd,MIXER_READ(SOUND_MIXER_SPEAKER), &vol);
      printf("speaker: \t%i\n",vol.left);
      ioctl(mixer_fd,MIXER_READ(SOUND_MIXER_CD), &vol);
      printf("cd: \t\t%i\n",vol.left);
      ioctl(mixer_fd,MIXER_READ(SOUND_MIXER_MIC), &vol);
      printf("mic: \t\t%i\n",vol.left);
      ioctl(mixer_fd,MIXER_READ(SOUND_MIXER_LINE), &vol);
      printf("line: \t\t%i\n",vol.left);
    }
    else if (!strcmp(argv[1],"mute")) 
    {
      vol.left=vol.right=0;
      ioctl(mixer_fd,MIXER_WRITE(SOUND_MIXER_VOLUME), &vol);
      ioctl(mixer_fd,MIXER_WRITE(SOUND_MIXER_PCM), &vol);
      ioctl(mixer_fd,MIXER_WRITE(SOUND_MIXER_CD), &vol);
      ioctl(mixer_fd,MIXER_WRITE(SOUND_MIXER_SPEAKER), &vol);
      ioctl(mixer_fd,MIXER_WRITE(SOUND_MIXER_LINE), &vol);
    }
    else if (argv[1][0]<48 || argv[1][0]>57) 
    {
      printf("option not recognized ... exiting\n");
      exit(1);
    } 
    else 
    {
      vol.left=vol.right=atoi(argv[1])%100;
      ioctl(mixer_fd,MIXER_WRITE(SOUND_MIXER_VOLUME), &vol);
    }
  } 

  return 0;
}



